# sage_setup: distribution = sagemath-repl
# sage.doctest: needs sage.graphs sage.numerical.mip
"""
This file (./sol/lp_doctest.sage) was *autogenerated* from ./sol/lp.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/lp_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/lp.tex, line 1::

  sage: sage.numerical.backends.generic_backend.default_solver = "Glpk"

Sage example in ./sol/lp.tex, line 28::

  sage: l = [28, 10, -89, 69, 42, -37, 76, 78, -40, 92, -93, 45]
  sage: p = MixedIntegerLinearProgram()
  sage: b = p.new_variable(binary = True)
  sage: p.add_constraint(p.sum([ v*b[v] for v in l ]) == 0)
  sage: p.add_constraint(p.sum([ b[v] for v in l ]) >= 1)
  sage: p.solve()
  0.0
  sage: b = p.get_values(b, convert=ZZ, tolerance=1e-3)
  sage: len([v for v in b if b[v] == 1])
  5
  sage: sum([v for v in b if b[v] == 1])
  0

Sage example in ./sol/lp.tex, line 64::

  sage: g = graphs.PetersenGraph()
  sage: p = MixedIntegerLinearProgram(maximization = False)
  sage: b = p.new_variable(binary = True)
  sage: for v in g:
  ....:     p.add_constraint( p.sum([b[u] for u in g.neighbors(v)])
  ....:                       + b[v] >= 1)
  sage: p.set_objective( p.sum([ b[v] for v in g ]) )
  sage: p.solve()
  3.0
  sage: b = p.get_values(b, convert=ZZ, tolerance=1e-3)
  sage: dom = [v for v in b if b[v] == 1]
  sage: len(dom)
  3
  sage: all((v in dom or any(g.has_edge(x,v) for x in dom)) for v in g)
  True
"""



# sage_setup: distribution = sagemath-repl
# sage.doctest: needs sage.symbolic
"""
C Function Profiler Using Google Perftools

Note that the profiler samples 100x per second by default. In
particular, you cannot profile anything shorter than 10ms. You can
adjust the rate with the ``CPUPROFILE_FREQUENCY`` environment variable
if you want to change it.

EXAMPLES::

    sage: from sage.misc.gperftools import Profiler, run_100ms
    sage: prof = Profiler()
    sage: prof.start()       # optional - gperftools
    sage: run_100ms()
    sage: prof.stop()        # optional - gperftools
    PROFILE: interrupts/evictions/bytes = ...

REFERENCE:

This uses the `Google performance analysis tools
<https://github.com/gperftools/gperftools>`_. Note that they are not
included in Sage, you have to install them yourself on your system.

AUTHORS:

- Volker Braun (2014-03-31): initial version
"""

# ****************************************************************************
#       Copyright (C) 2014 Volker Braun <vbraun.name@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************

import ctypes
import sys
import time

from sage.misc.cachefunc import cached_method
from sage.structure.sage_object import SageObject


libc = None
libprofiler = None


class Profiler(SageObject):

    def __init__(self, filename=None) -> None:
        """
        Interface to the gperftools profiler.

        INPUT:

        - ``filename`` -- string or ``None`` (default). The file name
          to log to. By default, a new temporary file is created.

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler
            sage: Profiler()
            Profiler logging to ...
        """
        if filename is None:
            from sage.misc.temporary_file import tmp_filename
            self._filename = tmp_filename(ext='.perf')
        else:
            self._filename = filename

    def filename(self) -> str:
        """
        Return the file name.

        OUTPUT: string

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler
            sage: prof = Profiler()
            sage: prof.filename()
            '.../tmp_....perf'
        """
        return self._filename

    def _repr_(self) -> str:
        """
        Return string representation.

        OUTPUT: string

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler
            sage: Profiler()
            Profiler logging to .../tmp....perf
        """
        return f'Profiler logging to {self.filename()}'

    def _libprofiler(self):
        """
        Return libprofiler.

        OUTPUT: a ctypes shared library handle

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler
            sage: Profiler()._libprofiler()    # optional - gperftools
            <CDLL 'libprofiler...', handle ... at ...>
        """
        global libprofiler
        if libprofiler is not None:
            return libprofiler
        import ctypes.util
        name = ctypes.util.find_library('profiler')
        if name:
            libprofiler = ctypes.CDLL(name)
            return libprofiler
        else:
            raise ImportError('failed to open libprofiler, make sure gperftools is installed')

    def start(self):
        """
        Start profiling.

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler, run_100ms
            sage: prof = Profiler()
            sage: prof.start()    # optional - gperftools
            sage: run_100ms()
            sage: prof.stop()     # optional - gperftools
            PROFILE: interrupts/evictions/bytes = ...
        """
        from signal import SIGPROF, SIG_DFL
        from cysignals.pysignals import setossignal
        self._previous_sigprof_handler = setossignal(SIGPROF, SIG_DFL)
        profiler = self._libprofiler()
        self._t_start = time.time()
        rc = profiler.ProfilerStart(str.encode(self.filename()))
        if rc < 0:
            raise ValueError('profiler failed to start')

    def stop(self):
        """
        Stop the CPU profiler.

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler, run_100ms
            sage: prof = Profiler()
            sage: prof.start()    # optional - gperftools
            sage: run_100ms()
            sage: prof.stop()     # optional - gperftools
            PROFILE: interrupts/evictions/bytes = ...
        """
        profiler = self._libprofiler()
        profiler.ProfilerStop()
        self._t_stop = time.time()
        if (self._t_stop - self._t_start) < 0.1:
            from warnings import warn
            warn('not enough samples, total runtime was '
                 'less than 100ms', RuntimeWarning)

    @cached_method
    def _pprof(self) -> str:
        """
        Return the name of the ``pprof`` binary.

        OUTPUT:

        String. The name of the gperftools ``pprof`` utility. A
        :exc:`OSError` is raised if it cannot be found.

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler
            sage: prof = Profiler()
            sage: try:
            ....:     pp = prof._pprof()
            ....:     assert isinstance(pp, str)
            ....: except OSError:
            ....:     pass    # not installed
        """
        potential_names = ['google-pprof', 'pprof']
        from subprocess import check_output, CalledProcessError, STDOUT
        for name in potential_names:
            try:
                bytes_version = check_output([name, '--version'], stderr=STDOUT)
            except (CalledProcessError, OSError):
                continue
            version = bytes_version.decode()
            if 'gperftools' not in version:
                from warnings import warn
                warn('the "{0}" utility does not appear to be the gperftools profiler'
                     .format(name), RuntimeWarning)
                continue
            return name
        raise OSError('unable to run pprof, please install gperftools')

    def _executable(self) -> str:
        """
        Return the name of the Sage Python interpreter.

        OUTPUT: string

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler
            sage: prof = Profiler()
            sage: prof._executable()
            '.../python...'
        """
        return sys.executable

    def _call_pprof(self, *args, **kwds):
        """
        Run the pprof binary.

        INPUT:

        - ``args`` -- list of strings; the arguments to ``pprof``

        - ``kwds`` -- keyword arguments passed to ``subprocess.check_call``

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler
            sage: prof = Profiler()
            sage: prof._call_pprof('--help')   # optional - gperftools
            Usage:
            pprof [options] <program> <profiles>
            ...
        """
        from subprocess import check_call
        check_call([self._pprof()] + list(args), **kwds)

    def top(self, cumulative=True):
        """
        Print text report.

        OUTPUT: nothing; a textual report is printed to stdout

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler
            sage: prof = Profiler()
            sage: prof.start()    # optional - gperftools
            sage: # do something
            sage: prof.stop()     # optional - gperftools
            PROFILE: interrupts/evictions/bytes = ...
            sage: prof.top()   # optional - gperftools
            Using local file ...
            Using local file ...
        """
        args = []
        if cumulative:
            args += ['--cum']
        args += ['--text', self._executable(), self.filename()]
        self._call_pprof(*args)

    def save(self, filename, cumulative=True, verbose=True):
        """
        Save report to disk.

        INPUT:

        - ``filename`` -- string; the filename to save at. Must end
          with one of ``.dot``, ``.ps``, ``.pdf``, ``.svg``, ``.gif``,
          or ``.txt`` to specify the output file format.

        - ``cumulative`` -- boolean (default: ``True``); whether to return
          cumulative timings

        - ``verbose`` -- boolean (default: ``True``); whether to print
          informational messages

        EXAMPLES::

            sage: from sage.misc.gperftools import Profiler, run_100ms
            sage: prof = Profiler()
            sage: prof.start()    # optional - gperftools
            sage: run_100ms()     # optional - gperftools
            sage: prof.stop()     # optional - gperftools
            PROFILE: interrupts/evictions/bytes = ...
            sage: f = tmp_filename(ext='.txt')      # optional - gperftools
            sage: prof.save(f, verbose=False)       # optional - gperftools
        """
        args = []
        if cumulative:
            args += ['--cum']
        if filename.endswith('.dot'):
            args += ['--dot']
        elif filename.endswith('.ps'):
            args += ['--ps']
        elif filename.endswith('.pdf'):
            args += ['--pdf']
        elif filename.endswith('.svg'):
            args += ['--svg']
        elif filename.endswith('.txt'):
            args += ['--text']
        elif filename.endswith('.gif'):
            args += ['--gif']
        else:
            raise ValueError('unknown extension')
        args += [self._executable(), self.filename()]
        stderr = sys.stdout if verbose else False
        with open(filename, 'wb') as outfile:
            self._call_pprof(*args, stdout=outfile, stderr=stderr)


def crun(s, evaluator):
    """
    Profile single statement.

    - ``s`` -- string; Sage code to profile

    - ``evaluator`` -- callable to evaluate

    EXAMPLES::

        sage: import sage.misc.gperftools as gperf
        sage: ev = lambda ex:eval(ex, globals(), locals())
        sage: gperf.crun('gperf.run_100ms()', evaluator=ev)   # optional - gperftools
        PROFILE: interrupts/evictions/bytes = ...
        Using local file ...
        Using local file ...
    """
    prof = Profiler()
    from sage.repl.preparse import preparse
    py_s = preparse(s)
    prof.start()
    try:
        evaluator(py_s)
    finally:
        prof.stop()
    prof.top()


def run_100ms() -> None:
    """
    Used for doctesting.

    A function that performs some computation for more than (but not
    that much more than) 100ms.

    EXAMPLES::

        sage: from sage.misc.gperftools import run_100ms
        sage: run_100ms()
    """
    t0 = time.time()   # start
    t1 = t0 + 0.1      # end
    from sage.symbolic.ring import SR
    while time.time() < t1:
        sum(1 / (1 + SR(n) ** 2) for n in range(100))



# sage_setup: distribution = sagemath-repl
r"""
Video Output Types

This module defines the rich output types for video formats.
"""
# ****************************************************************************
#       Copyright (C) 2015 Martin von Gagern <Martin.vGagern@gmx.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************
import importlib.resources

from sage.repl.rich_output.output_basic import OutputBase
from sage.repl.rich_output.buffer import OutputBuffer


class OutputVideoBase(OutputBase):

    def __init__(self, video, loop=True):
        """
        Abstract base class for rich video output.

        INPUT:

        - ``video`` -- :class:`~sage.repl.rich_output.buffer.OutputBuffer`;
          the video data
        - ``loop`` -- boolean; whether to repeat the video in an endless loop

        EXAMPLES::

            sage: from sage.repl.rich_output.output_catalog import OutputVideoOgg
            sage: OutputVideoOgg.example()  # indirect doctest
            OutputVideoOgg container
        """
        assert isinstance(video, OutputBuffer)
        self.video = video
        self.loop = loop

    @classmethod
    def example(cls):
        r"""
        Construct a sample video output container.

        This static method is meant for doctests, so they can easily
        construct an example.  The method is implemented in the abstract
        :class:`OutputVideoBase` class, but should get invoked on a
        concrete subclass for which an actual example can exist.

        OUTPUT: an instance of the class on which this method is called

        EXAMPLES::

            sage: from sage.repl.rich_output.output_catalog import OutputVideoOgg
            sage: OutputVideoOgg.example()
            OutputVideoOgg container
            sage: OutputVideoOgg.example().video
            buffer containing 5612 bytes
            sage: OutputVideoOgg.example().ext
            '.ogv'
            sage: OutputVideoOgg.example().mimetype
            'video/ogg'
        """
        with importlib.resources.path(__package__, 'example' + cls.ext) as filename:
            return cls(OutputBuffer.from_file(filename),
                       {'controls': True, 'loop': False})

    def html_fragment(self, url, link_attrs=''):
        r"""
        Construct a HTML fragment for embedding this video.

        INPUT:

        - ``url`` -- string; the URL where the data of this video can be found

        - ``link_attrs`` -- string; can be used to style the fallback link
          which is presented to the user if the video is not supported

        EXAMPLES::

            sage: from sage.repl.rich_output.output_catalog import OutputVideoOgg
            sage: print(OutputVideoOgg.example().html_fragment
            ....:       ('foo', 'class="bar"').replace('><','>\n<'))
            <video autoplay="autoplay" controls="controls" loop="loop">
            <source src="foo" type="video/ogg" />
            <p>
            <a target="_new" href="foo" class="bar">Download video/ogg video</a>
            </p>
            </video>
        """
        attrs = {
            'autoplay': 'autoplay',
            'controls': 'controls',
        }
        if self.loop:
            attrs['loop'] = 'loop'
        attrs = ''.join(' {}="{}"'.format(k, v)
                        for k, v in sorted(attrs.items()))
        txt = ('<video{attrs}>'
               '<source src="{url}" type="{mimetype}" /><p>'
               '<a target="_new" href="{url}" {link_attrs}>'
               'Download {mimetype} video</a></p></video>')
        return txt.format(url=url,
                          mimetype=self.mimetype,
                          attrs=attrs,
                          link_attrs=link_attrs)


class OutputVideoOgg(OutputVideoBase):
    """
    Ogg video, Ogg Theora in particular.

    EXAMPLES::

        sage: from sage.repl.rich_output.output_catalog import OutputVideoOgg
        sage: OutputVideoOgg.example()
        OutputVideoOgg container
    """

    ext = ".ogv"
    mimetype = "video/ogg"


class OutputVideoWebM(OutputVideoBase):
    """
    WebM video.

    The video can be encoded using VP8, VP9 or an even more recent codec.

    EXAMPLES::

        sage: from sage.repl.rich_output.output_catalog import OutputVideoWebM
        sage: OutputVideoWebM.example()
        OutputVideoWebM container
    """

    ext = ".webm"
    mimetype = "video/webm"


class OutputVideoMp4(OutputVideoBase):
    """
    MPEG 4 video.

    EXAMPLES::

        sage: from sage.repl.rich_output.output_catalog import OutputVideoMp4
        sage: OutputVideoMp4.example()
        OutputVideoMp4 container
    """

    ext = ".mp4"
    mimetype = "video/mp4"


class OutputVideoFlash(OutputVideoBase):
    """
    Flash video.

    EXAMPLES::

        sage: from sage.repl.rich_output.output_catalog import OutputVideoFlash
        sage: OutputVideoFlash.example()
        OutputVideoFlash container
    """

    ext = ".flv"
    mimetype = "video/x-flv"


class OutputVideoMatroska(OutputVideoBase):
    """
    Matroska Video.

    EXAMPLES::

        sage: from sage.repl.rich_output.output_catalog import OutputVideoMatroska
        sage: OutputVideoMatroska.example()
        OutputVideoMatroska container
    """

    ext = ".mkv"
    mimetype = "video/x-matroska"


class OutputVideoAvi(OutputVideoBase):
    """
    AVI video.

    EXAMPLES::

        sage: from sage.repl.rich_output.output_catalog import OutputVideoAvi
        sage: OutputVideoAvi.example()
        OutputVideoAvi container
    """

    ext = ".avi"
    mimetype = "video/x-msvideo"


class OutputVideoWmv(OutputVideoBase):
    """
    Windows Media Video.

    EXAMPLES::

        sage: from sage.repl.rich_output.output_catalog import OutputVideoWmv
        sage: OutputVideoWmv.example()
        OutputVideoWmv container
    """

    ext = ".wmv"
    mimetype = "video/x-ms-wmv"


class OutputVideoQuicktime(OutputVideoBase):
    """
    Quicktime video.

    EXAMPLES::

        sage: from sage.repl.rich_output.output_catalog import OutputVideoQuicktime
        sage: OutputVideoQuicktime.example()
        OutputVideoQuicktime container
    """

    ext = ".mov"
    mimetype = "video/quicktime"



# sage_setup: distribution = sagemath-repl
# sage.doctest: needs sage.libs.singular sage.rings.finite_rings sage.rings.number_field
##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Section 16.9 Sage
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: F = QuadraticField(7)
    sage: F
    Number Field in a with defining polynomial x^2 - 7 with a = 2.645751311064591?

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: root = F.gen(0)
    sage: root^2
    7

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: root
    a

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: (2*root)^3
    56*a

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: F.<b> = QuadraticField(7)
    sage: F
    Number Field in b with defining polynomial x^2 - 7 with b = 2.645751311064591?

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b^2
    7

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: (2*b)^3
    56*b

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: C.<t> = CyclotomicField(8)
    sage: C.random_element()   # random
    -2/11*t^2 + t - 1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z7 = Integers(7)
    sage: Z9 = Integers(9)
    sage: Q = QuadraticField(-11)
    sage: F.<a> = FiniteField(3^2)
    sage: P.<x> = Z7[]
    sage: S.<f,g,h> = QuaternionAlgebra(-7, 3)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: QQ.is_exact()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: RR.is_exact()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z7.is_finite()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z7.is_finite()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z7.is_integral_domain()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z9.is_integral_domain()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z9.is_field()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: F.is_field()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.is_field()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.is_commutative()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: S.is_commutative()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z7.characteristic()
    7

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z9.characteristic()
    9

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.characteristic()
    0

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: F.characteristic()
    3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: P.characteristic()
    7

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: S.characteristic()
    0

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b = Z9.zero(); b
    0

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b.parent()
    Ring of integers modulo 9

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: c = Q.zero(); c
    0

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: c.parent()
    Number Field in a with defining polynomial x^2 + 11 with a = 3.316624790355400?*I

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b == c
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: d = Z9.one(); d
    1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: d.parent()
    Ring of integers modulo 9

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: e = Q.one(); e
    1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: e.parent()
    Number Field in a with defining polynomial x^2 + 11 with a = 3.316624790355400?*I

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: d == e
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: QQ.is_subring(Q)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: QQ.is_subring(S)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: QQ.is_subring(F)
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: three = Z9(3)
    sage: three.is_unit()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: three*three
    0

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: four = Z9(4)
    sage: four.is_unit()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: g = four^-1; g
    7

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: four*g
    1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: I1 = ZZ.ideal(4)
    sage: I2 = 4*ZZ
    sage: I3 = (-4)*ZZ
    sage: I1 == I2
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: I2 == I3
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q = ZZ.quotient(I1); Q
    Ring of integers modulo 4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q == Integers(4)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z7 = Integers(7)
    sage: P.<y> = Z7[]
    sage: M = P.ideal(y^2+4)
    sage: Q = P.quotient(M)
    sage: Q
    Univariate Quotient Polynomial Ring in ybar over
    Ring of integers modulo 7 with modulus y^2 + 4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.random_element()   # random
    2*ybar + 6

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.order()
    49

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.is_field()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.<t> = P.quotient(M); Q
    Univariate Quotient Polynomial Ring in t over
    Ring of integers modulo 7 with modulus y^2 + 4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.random_element()   # random
    4*t + 6

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z7 = Integers(7)
    sage: P.<y> = Z7[]
    sage: M = P.ideal(y^2+3)
    sage: Q.<t> = P.quotient(M)
    sage: Q
    Univariate Quotient Polynomial Ring in t over
    Ring of integers modulo 7 with modulus y^2 + 3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.random_element()   # random
    3*t + 1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.order()
    49

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.is_field()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z7 = Integers(7)
    sage: P.<y> = Z7[]
    sage: M = P.ideal(y^2+4)
    sage: N = P.ideal(y^2+3)
    sage: M.is_maximal()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: N.is_maximal()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: M.is_prime()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: N.is_prime()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: H = Hom(ZZ, QQ)
    sage: phi = H([1])
    sage: phi
    Ring morphism:
      From: Integer Ring
      To:   Rational Field
      Defn: 1 |--> 1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: phi.parent()
    Set of Homomorphisms from Integer Ring to Rational Field

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 3; a
    3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a.parent()
    Integer Ring

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b = phi(3); b
    3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b.parent()
    Rational Field
"""



# sage_setup: distribution = sagemath-repl
# sage.doctest: needs fpylll sage.rings.finite_rings sage.rings.number_field sage.symbolic
##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Section 18.5 Sage
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q = ZZ.fraction_field(); Q
    Rational Field

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q == QQ
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: R.<x> = ZZ[]
    sage: P = R.fraction_field();P
    Fraction Field of Univariate Polynomial Ring in x over Integer Ring

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: f = P((x^2+3)/(7*x+4))
    sage: g = P((4*x^2)/(3*x^2-5*x+4))
    sage: h = P((-2*x^3+4*x^2+3)/(x^2+1))
    sage: ((f+g)/h).numerator()
    3*x^6 + 23*x^5 + 32*x^4 + 8*x^3 + 41*x^2 - 15*x + 12

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: ((f+g)/h).denominator()
    -42*x^6 + 130*x^5 - 108*x^4 + 63*x^3 - 5*x^2 + 24*x + 48

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: F.<c> = FiniteField(3^5)
    sage: F.characteristic()
    3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = F.prime_subfield(); G
    Finite Field of size 3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G.list()
    [0, 1, 2]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: K.<y>=QuadraticField(-7); K
    Number Field in y with defining polynomial x^2 + 7 with y = 2.645751311064591?*I

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: K.prime_subfield()
    Rational Field

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: K.<x> = ZZ[sqrt(-3)]; K
    Order of conductor 2 generated by a in Number Field in a
     with defining polynomial x^2 + 3 with a = 0.?e-18 + 1.732050807568878?*I

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: K.is_integral_domain()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: K.basis()
    [1, a]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: x
    a

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: (1+x)*(1-x) == 2*2
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: four = K(4)
    sage: four.is_unit()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: four^-1
    1/4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: T.<x>=ZZ[]
    sage: T.is_integral_domain()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: J = T.ideal(5, x); J
    Ideal (5, x) of Univariate Polynomial Ring in x over Integer Ring

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q = T.quotient(J); Q
    Quotient of Univariate Polynomial Ring in x over
    Integer Ring by the ideal (5, x)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: J.is_principal()
    Traceback (most recent call last):
    ...
    NotImplementedError

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Q.is_field()
    Traceback (most recent call last):
    ...
    NotImplementedError
"""
